@@ -5,8 +5,8 @@ from sqlobject.main import deprecated
 MySQLdb = None
 
 class ErrorMessage(str):
-    def __new__(cls, e):
-        obj = str.__new__(cls, e[1])
+    def __new__(cls, e, append_msg=''):
+        obj = str.__new__(cls, e[1] + append_msg)
         obj.code = int(e[0])
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
@@ -49,6 +49,12 @@ class MySQLConnection(DBAPI):
         if "sqlobject_encoding" in kw:
             del kw["sqlobject_encoding"]
             deprecated("sqlobject_encoding is deprecated and no longer used.")
+
+        # MySQLdb < 1.2.1: only ascii
+        # MySQLdb = 1.2.1: only unicode
+        # MySQLdb > 1.2.1: both ascii and unicode
+        self.need_unicode = (MySQLdb.version_info[:3] >= (1, 2, 1)) and (MySQLdb.version_info[:3] < (1, 2, 2))
+
         DBAPI.__init__(self, **kw)
 
     def connectionFromURI(cls, uri):
@@ -72,10 +78,8 @@ class MySQLConnection(DBAPI):
             if MySQLdb.version_info[:3] >= (1, 2, 2):
                 conn.ping(True) # Attempt to reconnect. This setting is persistent.
         except self.module.OperationalError, e:
-            raise OperationalError(
-                "%s; used connection string: host=%s, port=%s, db=%s, user=%s, pwd=%s" % (
-                e, self.host, self.port, self.db, self.user, self.password)
-            )
+            conninfo = "; used connection string: host=%(host)s, port=%(port)s, db=%(db)s, user=%(user)s" % self.__dict__
+            raise OperationalError(ErrorMessage(e, conninfo))
 
         if hasattr(conn, 'autocommit'):
             conn.autocommit(bool(self.autoCommit))
@@ -94,6 +98,12 @@ class MySQLConnection(DBAPI):
             conn.autocommit(auto)
 
     def _executeRetry(self, conn, cursor, query):
+        if self.need_unicode and not isinstance(query, unicode):
+            try:
+                query = unicode(query, self.dbEncoding)
+            except UnicodeError:
+                pass
+
         # When a server connection is lost and a query is attempted, most of
         # the time the query will raise a SERVER_LOST exception, then at the
         # second attempt to execute it, the mysql lib will reconnect and
